use linked_list_allocator::LockedHeap;
use spin::Mutex;
use tetris::TetrisBoard;
use vga::Vga13h;

use self::memory_map::memory_map;
use self::thunk::ThunkData;

use crate::println;

mod memory_map;
mod panic;
mod tetris;
mod thunk;
mod vga;

// Real mode memory allocation, for use with thunk
// 0x500 to 0x7BFF is free
const MEMORY_MAP_ADDR: usize = 0x1380; // 24 bytes, ends at 0x1397
const THUNK_STACK_ADDR: usize = 0x7C00; // Grows downwards

#[global_allocator]
static ALLOCATOR: LockedHeap = LockedHeap::empty();

pub(crate) static VGA: Mutex<Vga13h> = Mutex::new(unsafe { Vga13h::new() });

#[no_mangle]
pub unsafe extern "C" fn start(
	_boot_disk: usize,
	thunk10: extern "C" fn(),
	_thunk13: extern "C" fn(),
	thunk15: extern "C" fn(),
	_thunk16: extern "C" fn(),
) -> ! {
	println!("Entered Rust, serial ready.");

	{
		// Set vga mode
		let mut data = ThunkData::new();
		data.eax = 0x13;
		data.with(thunk10);
	}

	{
		// Disable cursor
		let mut data = ThunkData::new();
		data.eax = 0x0100;
		data.ecx = 0x3F00;
		data.with(thunk10);
	}

	// Clear screen
	let mut v = VGA.lock();

	/*
	let fb = v.get_fb();
	let mut c: u8 = 0;
	for x in 0..128 {
		for y in 0..128 {
			let idx = Vga13h::pix_idx(x, y);
			fb[idx] = c;
			c = c.wrapping_add(1);
		}
	}
	*/

	let t = TetrisBoard::new();
	t.draw(&mut v);

	let (heap_start, heap_size) = memory_map(thunk15).expect("No memory for heap");

	ALLOCATOR.lock().init(heap_start as *mut u8, heap_size);

	panic!("kernel");
}
